#ifndef CLASS_PLAIN_PARSER_H
#define CLASS_PLAIN_PARSER_H

#define CLASSPLAIN_ABIVERSION_MINOR 57
#define CLASSPLAIN_ABIVERSION_MAJOR 0

#define CLASSPLAIN_ABIVERSION  ((CLASSPLAIN_ABIVERSION_MAJOR << 16) | (CLASSPLAIN_ABIVERSION_MINOR))

/* Function prototypes */

OP *ClassPlain_newMETHSTARTOP(pTHX_ U32 flags);

OP *ClassPlain_newCOMMONMETHSTARTOP(pTHX_ U32 flags);

#endif
